@@ -45,18 +45,14 @@ SOFTWARE.
 
 ******************************************************************/
 
+#include "pixmanint.h"
+
 #include <stdlib.h>
 #include <limits.h>
 #include <string.h>
 
 #include "pixregionint.h"
 
-#if defined (__GNUC__) && !defined (NO_INLINES)
-#define INLINE	__inline
-#else
-#define INLINE
-#endif
-
 #undef assert
 #ifdef DEBUG_PIXREGION
 #define assert(expr) {if (!(expr)) \
@@ -77,16 +73,11 @@ static pixman_box16_t pixman_region_emptyBox = {0, 0, 0, 0};
 static pixman_region16_data_t pixman_region_emptyData = {0, 0};
 
 static pixman_region16_data_t  pixman_brokendata = {0, 0};
-static pixman_region16_t   pixman_brokenregion = { { 0, 0, 0, 0 }, &pixman_brokendata };
 
 static pixman_region_status_t
 pixman_break (pixman_region16_t *pReg);
-
-static void
-pixman_init (pixman_region16_t *region, pixman_box16_t *rect);
-
-static void
-pixman_uninit (pixman_region16_t *region);
+static pixman_region_status_t
+pixman_rect_alloc(pixman_region16_t * region, int n);
 
 /*
  * The functions in this file implement the Region abstraction used extensively
@@ -157,7 +148,31 @@ pixman_uninit (pixman_region16_t *region);
         ((r1)->y1 <= (r2)->y1) && \
         ((r1)->y2 >= (r2)->y2) )
 
-#define allocData(n) malloc(PIXREGION_SZOF(n))
+static size_t
+PIXREGION_SZOF(size_t n)
+{
+    size_t size = n * sizeof(pixman_box16_t);
+    if (n > UINT32_MAX / sizeof(pixman_box16_t))
+	return 0;
+
+    if (sizeof(pixman_region16_data_t) > UINT32_MAX - size)
+	return 0;
+
+    return size + sizeof(pixman_region16_data_t);
+}
+
+static void
+allocData(size_t n, void **data)
+{
+    size_t sz = PIXREGION_SZOF(n);
+    if (!sz) {
+	*data = NULL;
+	return;
+    }
+
+    *data = malloc(sz);
+}
+
 #define freeData(reg) if ((reg)->data && (reg)->data->size) free((reg)->data)
 
 #define RECTALLOC_BAIL(pReg,n,bail) \
@@ -194,7 +209,10 @@ if (!(pReg)->data || (((pReg)->data->numRects + (n)) > (pReg)->data->size)) \
 if (((numRects) < ((reg)->data->size >> 1)) && ((reg)->data->size > 50)) \
 {									 \
     pixman_region16_data_t * NewData;							 \
-    NewData = (pixman_region16_data_t *)realloc((reg)->data, PIXREGION_SZOF(numRects));	 \
+    size_t data_size = PIXREGION_SZOF(numRects);			 \
+    if (!data_size)							 \
+      goto bail;							 \
+    NewData = (pixman_region16_data_t *)realloc((reg)->data, data_size); \
     if (NewData)							 \
     {									 \
 	NewData->size = (numRects);					 \
@@ -297,66 +315,63 @@ pixman_region16_valid(reg)
 
 #endif /* DEBUG_PIXREGION */
 
-/*	Create a new empty region	*/
-pixman_region16_t *
-pixman_region_create (void)
+void
+pixman_region_init(pixman_region16_t *region)
 {
-    return pixman_region_create_simple (NULL);
-}
-
-/*****************************************************************
- *   pixman_region_create_simple (extents)
- *     This routine creates a pixman_region16_t for a simple
- *     rectangular region.
- *****************************************************************/
-pixman_region16_t *
-pixman_region_create_simple (pixman_box16_t *extents)
-{
-    pixman_region16_t *region;
-
-    region = malloc (sizeof (pixman_region16_t));
-    if (region == NULL)
-	return &pixman_brokenregion;
-
-    pixman_init (region, extents);
-
-    return region;
-}
-
-/*****************************************************************
- *   RegionInit(pReg, rect, size)
- *     Outer region rect is statically allocated.
- *****************************************************************/
-
-static void
-pixman_init(pixman_region16_t *region, pixman_box16_t *extents)
-{
-    if (extents)
-    {
-	region->extents = *extents;
-	region->data = NULL;
-    }
-    else
-    {
-	region->extents = pixman_region_emptyBox;
-	region->data = &pixman_region_emptyData;
-    }
-}
-
-static void
-pixman_uninit (pixman_region16_t *region)
-{
-    good (region);
-    freeData (region);
+    region->extents = pixman_region_emptyBox;
+    region->data = &pixman_region_emptyData;
 }
 
 void
-pixman_region_destroy (pixman_region16_t *region)
+pixman_region_init_rect(pixman_region16_t *region,
+                        int x, int y, unsigned int width, unsigned int height)
 {
-    pixman_uninit (region);
+    region->extents.x1 = x;
+    region->extents.y1 = y;
+    region->extents.x2 = x + width;
+    region->extents.y2 = y + height;
+    region->data = NULL;
+}
 
-    if (region != &pixman_brokenregion)
-	free (region);
+void
+pixman_region_init_with_extents(pixman_region16_t *region, pixman_box16_t *extents)
+{
+    region->extents = *extents;
+    region->data = NULL;
+}
+
+pixman_region_status_t
+pixman_region_init_rects(pixman_region16_t *region, pixman_box16_t *boxes, int count)
+{
+    int overlap;
+
+    if (count == 1) {
+	pixman_region_init_rect(region,
+				boxes[0].x1,
+				boxes[0].y1,
+				boxes[0].x2 - boxes[0].x1,
+				boxes[0].y2 - boxes[0].y1);
+	return PIXMAN_REGION_STATUS_SUCCESS;
+    }
+
+    pixman_region_init(region);
+    if (!pixman_rect_alloc(region, count))
+	return PIXMAN_REGION_STATUS_FAILURE;
+
+    /* Copy in the rects */
+    memcpy (PIXREGION_RECTS(region), boxes, sizeof(pixman_box16_t) * count);
+    region->data->numRects = count;
+
+    /* Validate */
+    region->extents.x1 = region->extents.x2 = 0;
+    return pixman_region_validate (region, &overlap);
+}
+
+void
+pixman_region_fini (pixman_region16_t *region)
+{
+    good (region);
+    freeData (region);
 }
 
 int
@@ -384,11 +399,12 @@ static pixman_region_status_t
 pixman_rect_alloc(pixman_region16_t * region, int n)
 {
     pixman_region16_data_t *data;
+    size_t data_size;
 
     if (!region->data)
     {
 	n++;
-	region->data = allocData(n);
+	allocData(n, (void **) &region->data);
 	if (!region->data)
 	    return pixman_break (region);
 	region->data->numRects = 1;
@@ -396,7 +412,7 @@ pixman_rect_alloc(pixman_region16_t * region, int n)
     }
     else if (!region->data->size)
     {
-	region->data = allocData(n);
+      allocData(n, (void **) &region->data);
 	if (!region->data)
 	    return pixman_break (region);
 	region->data->numRects = 0;
@@ -410,7 +426,10 @@ pixman_rect_alloc(pixman_region16_t * region, int n)
 		n = 250;
 	}
 	n += region->data->numRects;
-	data = (pixman_region16_data_t *)realloc(region->data, PIXREGION_SZOF(n));
+	data_size = PIXREGION_SZOF(n);
+	if (!data_size)
+	    return pixman_break (region);
+	data = (pixman_region16_data_t *)realloc(region->data, data_size);
 	if (!data)
 	    return pixman_break (region);
 	region->data = data;
@@ -436,7 +455,7 @@ pixman_region_copy(pixman_region16_t *dst, pixman_region16_t *src)
     if (!dst->data || (dst->data->size < src->data->numRects))
     {
 	freeData(dst);
-	dst->data = allocData(src->data->numRects);
+	allocData(src->data->numRects, (void **) &dst->data);
 	if (!dst->data)
 	    return pixman_break (dst);
 	dst->data->size = src->data->numRects;
@@ -469,7 +488,7 @@ pixman_region_copy(pixman_region16_t *dst, pixman_region16_t *src)
  *
  *-----------------------------------------------------------------------
  */
-INLINE static int
+static inline int
 pixman_coalesce (
     pixman_region16_t *	region,	    	/* Region to coalesce		     */
     int	    	  	prevStart,  	/* Index of start of previous band   */
@@ -552,7 +571,7 @@ pixman_coalesce (
  *-----------------------------------------------------------------------
  */
 
-INLINE static pixman_region_status_t
+static inline pixman_region_status_t
 pixman_region_appendNonO (
     pixman_region16_t *	region,
     pixman_box16_t *	r,
@@ -594,7 +613,7 @@ pixman_region_appendNonO (
 {									\
     int newRects;							\
     if ((newRects = rEnd - r)) {					\
-	RECTALLOC(newReg, newRects);					\
+	RECTALLOC_BAIL(newReg, newRects, bail);				\
 	memmove((char *)PIXREGION_TOP(newReg),(char *)r, 			\
               newRects * sizeof(pixman_box16_t));				\
 	newReg->data->numRects += newRects;				\
@@ -711,9 +730,13 @@ pixman_op(
 	newReg->data = &pixman_region_emptyData;
     else if (newReg->data->size)
 	newReg->data->numRects = 0;
-    if (newSize > newReg->data->size)
-	if (!pixman_rect_alloc(newReg, newSize))
+    if (newSize > newReg->data->size) {
+	if (!pixman_rect_alloc(newReg, newSize)) {
+	    if (oldData)
+		free (oldData);
 	    return PIXMAN_REGION_STATUS_FAILURE;
+	}
+    }
 
     /*
      * Initialize ybot.
@@ -770,7 +793,8 @@ pixman_op(
 		bot = MIN(r1->y2, r2y1);
 		if (top != bot)	{
 		    curBand = newReg->data->numRects;
-		    pixman_region_appendNonO(newReg, r1, r1BandEnd, top, bot);
+		    if (!pixman_region_appendNonO(newReg, r1, r1BandEnd, top, bot))
+			goto bail;
 		    Coalesce(newReg, prevBand, curBand);
 		}
 	    }
@@ -781,7 +805,8 @@ pixman_op(
 		bot = MIN(r2->y2, r1y1);
 		if (top != bot) {
 		    curBand = newReg->data->numRects;
-		    pixman_region_appendNonO(newReg, r2, r2BandEnd, top, bot);
+		    if (!pixman_region_appendNonO(newReg, r2, r2BandEnd, top, bot))
+			goto bail;
 		    Coalesce(newReg, prevBand, curBand);
 		}
 	    }
@@ -797,8 +822,9 @@ pixman_op(
 	ybot = MIN(r1->y2, r2->y2);
 	if (ybot > ytop) {
 	    curBand = newReg->data->numRects;
-	    (* overlapFunc)(newReg, r1, r1BandEnd, r2, r2BandEnd, ytop, ybot,
-			    pOverlap);
+	    if (!(* overlapFunc)(newReg, r1, r1BandEnd, r2, r2BandEnd, ytop, ybot,
+			    pOverlap))
+		goto bail;
 	    Coalesce(newReg, prevBand, curBand);
 	}
 
@@ -823,7 +849,8 @@ pixman_op(
 	/* Do first nonOverlap1Func call, which may be able to coalesce */
 	FindBand(r1, r1BandEnd, r1End, r1y1);
 	curBand = newReg->data->numRects;
-	pixman_region_appendNonO(newReg, r1, r1BandEnd, MAX(r1y1, ybot), r1->y2);
+	if (!pixman_region_appendNonO(newReg, r1, r1BandEnd, MAX(r1y1, ybot), r1->y2))
+	    goto bail;
 	Coalesce(newReg, prevBand, curBand);
 	/* Just append the rest of the boxes  */
 	AppendRegions(newReg, r1BandEnd, r1End);
@@ -832,14 +859,17 @@ pixman_op(
 	/* Do first nonOverlap2Func call, which may be able to coalesce */
 	FindBand(r2, r2BandEnd, r2End, r2y1);
 	curBand = newReg->data->numRects;
-	pixman_region_appendNonO(newReg, r2, r2BandEnd, MAX(r2y1, ybot), r2->y2);
+	if (!pixman_region_appendNonO(newReg, r2, r2BandEnd, MAX(r2y1, ybot), r2->y2))
+	    goto bail;
 	Coalesce(newReg, prevBand, curBand);
 	/* Append rest of boxes */
 	AppendRegions(newReg, r2BandEnd, r2End);
     }
 
-    if (oldData)
+    if (oldData) {
 	free(oldData);
+	oldData = NULL;
+    }
 
     if (!(numRects = newReg->data->numRects))
     {
@@ -858,6 +888,11 @@ pixman_op(
     }
 
     return PIXMAN_REGION_STATUS_SUCCESS;
+
+bail:
+    if (oldData)
+	free(oldData);
+    return PIXMAN_REGION_STATUS_FAILURE;
 }
 
 /*-
@@ -1137,7 +1172,7 @@ pixman_region_unionO (
 /* Convenience function for performing union of region with a single rectangle */
 pixman_region_status_t
 pixman_region_union_rect(pixman_region16_t *dest, pixman_region16_t *source,
-		   int x, int y, unsigned int width, unsigned int height)
+                         int x, int y, unsigned int width, unsigned int height)
 {
     pixman_region16_t region;
 
@@ -1421,7 +1456,7 @@ QuickSortRects(
  *      Step 2. Split the rectangles into the minimum number of proper y-x
  *		banded regions.  This may require horizontally merging
  *		rectangles, and vertically coalescing bands.  With any luck,
- *		this step in an identity tranformation (ala the Box widget),
+ *		this step in an identity transformation (ala the Box widget),
  *		or a coalescing into 1 box (ala Menus).
  *
  *	Step 3. Merge the separate regions down to a single region by calling
@@ -1491,7 +1526,7 @@ pixman_region_validate(pixman_region16_t * badreg,
 
     /* Set up the first region to be the first rectangle in badreg */
     /* Note that step 2 code will never overflow the ri[0].reg rects array */
-    ri = (RegionInfo *) malloc(4 * sizeof(RegionInfo));
+    ri = (RegionInfo *) _pixman_malloc_ab(4, sizeof(RegionInfo));
     if (!ri)
 	return pixman_break (badreg);
     sizeRI = 4;
@@ -1553,9 +1588,13 @@ pixman_region_validate(pixman_region16_t * badreg,
 	/* Uh-oh.  No regions were appropriate.  Create a new one. */
 	if (sizeRI == numRI)
 	{
+	    size_t data_size;
 	    /* Oops, allocate space for new region information */
 	    sizeRI <<= 1;
-	    rit = (RegionInfo *) realloc(ri, sizeRI * sizeof(RegionInfo));
+	    data_size = sizeRI * sizeof(RegionInfo);
+	    if (data_size / sizeRI != sizeof(RegionInfo))
+		goto bail;
+	    rit = (RegionInfo *) realloc(ri, data_size);
 	    if (!rit)
 		goto bail;
 	    ri = rit;
@@ -1657,7 +1696,7 @@ pixman_region_rectsToRegion(nrects, prect, ctype)
 	}
 	return region;
     }
-    pData = allocData(nrects);
+    allocData(nrects, &pData);
     if (!pData)
     {
 	pixman_break (region);
@@ -2165,7 +2204,7 @@ pixman_region16_data_copy(pixman_region16_t * dst, pixman_region16_t * src)
     if (!dst->data || (dst->data->size < src->data->numRects))
     {
 	freeData(dst);
-	dst->data = allocData(src->data->numRects);
+	allocData(src->data->numRects, &dst->data);
 	if (!dst->data)
 	    return pixman_break (dst);
     }
